// generated file, do not edit
#ifndef DLT_ID_MAPP_H
#define DLT_ID_MAPP_H

#define DLT_MSG_ID_MAPP_MCT1_1 1000
#define DLT_MSG_ID_MAPP_MCT2_1 1001
#define DLT_MSG_ID_MAPP_MCT3_1 1002
#define DLT_MSG_ID_MAPP_MCT1_2 1003

#endif /* DLT_ID_MAPP_H */
