/**
 * @licence app begin@
 * Copyright (C) 2011-2012  BMW AG
 *
 * This file is part of COVESA Project Dlt Viewer.
 *
 * Contributions are licensed to the COVESA Alliance under one or more
 * Contribution License Agreements.
 *
 * \copyright
 * This Source Code Form is subject to the terms of the
 * Mozilla Public License, v. 2.0. If a  copy of the MPL was not distributed with
 * this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * \author Christian Muck <christian.muck@bmw.de> 2011-2012
 *
 * \file mainpage.h
 * For further information see http://www.covesa.global/.
 * @licence end@
 */



#ifndef MAINPAGE_H_
#define MAINPAGE_H_

#endif /* MAINPAGE_H_ */

