// MSVC, Windows SDK
#if defined(_WIN32)

#include "../version.h"

#define STRINGIFY(x) #x
#define TOSTRING(x) STRINGIFY(x)

#include <winres.h>

LANGUAGE LANG_ENGLISH, SUBLANG_DEFAULT

VS_VERSION_INFO VERSIONINFO
    FILEVERSION PACKAGE_MAJOR_VERSION,PACKAGE_MINOR_VERSION,PACKAGE_PATCH_LEVEL,0
    PRODUCTVERSION PACKAGE_MAJOR_VERSION,PACKAGE_MINOR_VERSION,PACKAGE_PATCH_LEVEL,0
    FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
    FILEFLAGS 0x1L
#else
    FILEFLAGS 0x0L
#endif
    FILEOS 0x4L
    FILETYPE 0x1L
    FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "000904b0"
        BEGIN
            VALUE "FileDescription", PACKAGE_DESCRIPTION
            VALUE "LegalCopyright", "Copyright (c) BMW AG"
            VALUE "FileVersion", TOSTRING(PACKAGE_MAJOR_VERSION) "." TOSTRING(PACKAGE_MINOR_VERSION) "." TOSTRING(PACKAGE_PATCH_LEVEL) ".0"
/*
            VALUE "InternalName", PRODUCT_INTERNAL_NAME
            VALUE "OriginalFilename", PRODUCT_ORIGINAL_FILENAME
*/
            VALUE "ProductName", PACKAGE_DESCRIPTION
            VALUE "ProductVersion", TOSTRING(PACKAGE_MAJOR_VERSION) "." TOSTRING(PACKAGE_MINOR_VERSION) "." TOSTRING(PACKAGE_PATCH_LEVEL) ".0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x9, 1200
    END
END
#endif
