/**
 * @licence app begin@
 * Copyright (C) 2011-2023  BMW AG
 *
 * This file is part of COVESA Project Dlt Viewer.
 *
 * Contributions are licensed to the COVESA Alliance under one or more
 * Contribution License Agreements.
 *
 * \copyright
 * This Source Code Form is subject to the terms of the
 * Mozilla Public License, v. 2.0. If a  copy of the MPL was not distributed with
 * this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * \file version.h
 * For further information see http://www.covesa.global/.
 * @licence end@
 */

#ifndef __VERSION_H_
#define __VERSION_H_
/* version change recommendation */
/* changing minor & major when layout of settings file config.ini changes */
/* this kind of change is tracked in the settings dialogr */
/* for other bugfixes and not major feature enhancement just use patch level */
#define PACKAGE_VERSION_STATE "unstable"
#define PACKAGE_MAJOR_VERSION 2
#define PACKAGE_MINOR_VERSION 30
#define PACKAGE_PATCH_LEVEL 0
#define PACKAGE_VERSION "2.30.0"
#define PACKAGE_REVISION "unstable"
#define PACKAGE_DESCRIPTION "DLT Viewer"
#define DLT_SUPPORT_MAIL_ADDRESS ""
#define DLT_SUPPORT_NAME "COVESA"
#endif
