// generated file, do not edit
#ifndef DLT_ID_CAPP_H
#define DLT_ID_CAPP_H

#define DLT_MSG_ID_CAPP_CCT1_944ef165 1000
#define DLT_MSG_ID_CAPP_CCT2_4451f8f2 1001
#define DLT_MSG_ID_CAPP_CCT3_545fc245 1002
#define DLT_MSG_ID_CAPP_CCT1_387ba9d6 1003

#endif /* DLT_ID_CAPP_H */
